<?php

namespace FM\Service;

use FM\Factory\ApontamentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;

class ApontamentoService {

    protected $ApontamentoFactory;
    protected $FuncionariosHierarqService;
    protected $Environment;
    protected $newCode;
    
    public function __construct (EntityManager $entityManager, ApontamentoFactory $ApontamentoFactory, FuncionariosHierarqService $FuncionariosHierarqService, Environment $Environment, NewCode $newCode) {
        $this->connection                   = $entityManager->getConnection();
        $this->ApontamentoFactory           = $ApontamentoFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    public function getDadosApontamento() {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        return $this->ApontamentoFactory->getDadosApontamento($listaOperadores);
    }

    public function getDadosApontamentoInserido($param) {

        $nr = $param[0]["NRAPONTAMENTO"];
        $dados = $this->ApontamentoFactory->getDadosApontamentoInserido($nr);
        return $dados;
    }
    
    public function getDadosDocEObservacoes($param) {

        $nr = array(
            "NRAPONTAMENTO" => $param[0],
        );
        $dados = $this->ApontamentoFactory->getDadosObservacoes($nr);
        return $dados;
    }

    public function getIntervaloTrabalho($param) {
        $nr = array(
            "CDOPERADOR" => $param["CDOPERADOR"],
        );
        $dados = $this->ApontamentoFactory->getIntervaloTrabalho($nr);
        return $dados;
    }
    
    public function buscaRegistrosHora($param) {
        $cdoperador = $param["CDOPERADOR"];
        
        $dados = $this->ApontamentoFactory->buscaRegistrosHora($cdoperador);
        return $dados;
    }
    
    public function insertNovoApontamento($param){
      
        try {
            $nr = '';
            $NRORG =  $this->Environment->getNrOrg();
            
             $paramentroConsultaApontamento = array(
              "DTAPONTAMENTO" => $param["DATA"],
              "CDOPERADOR" => $param["CDOPERADOR"]
            );

            $consultaApontamento = $this->ApontamentoFactory->consultaApontamentoDia($paramentroConsultaApontamento);

            $codRegistroHoras = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

            $nrApontamento = $this->newCode->getCodeByTableName("FCM_APONTAMENTO", $NRORG, 1, 10, true);

            if (sizeof($consultaApontamento) === 0){
                
                $dadosInsertApontamento = array(
                        "NRAPONTAMENTO" => $nrApontamento, 
                        "DTAPONTAMENTO" => $param["DATA"],
                        "CDOPERADOR" => $param["CDOPERADOR"],
                        "HRENTRADA" => "",
                        "HRSAIDA" => "", 
                        "HRINTERVALO" => "", 
                        "HRTRABALHADAS" => "", 
                        "HRAUSENCIA" => "", 
                        "HRTOTAIS" => "", 
                        "IDFALTA" => "N", 
                        "IDAPURADO" => "N", 
                        "NRORG" => $NRORG, 
                        "IDATIVO" =>  "S"
                    );
                    
                $this->ApontamentoFactory->insertNovoApontamento($dadosInsertApontamento);
                   
                $lastEntry = $this->ApontamentoFactory->lastEntry($param["CDOPERADOR"]);
                    
                if (!empty($lastEntry)) {
                    $dateEntry = $lastEntry[0]['DTHRENTRADA'];
                } else {
                    $dateEntry = null;
                }
                
                if ($param["IDTIPORESGISTRO"] === "1" && $param["IDALMOCO"] === 0) {
                    $entrada = "{$param["DATA"]} {$param["HORA"]}";
                    $idalmoco = "N";
                    $tipo = "E";
                } else if ($param["IDTIPORESGISTRO"] === "1" && $param["IDALMOCO"] === 1){
                    $entrada = "{$param["DATA"]} {$param["HORA"]}";
                    $idalmoco = "S";
                    $tipo = "E";
                } else if ($param["IDTIPORESGISTRO"] === "2" && $param["IDALMOCO"] === 0) {
                    $saida = "{$param["DATA"]} {$param["HORA"]}";
                    $idalmoco = "N";
                    $tipo = "S";
                } else if ($param["IDTIPORESGISTRO"] === "2" && $param["IDALMOCO"] === 1){
                    $saida = "{$param["DATA"]} {$param["HORA"]}";
                    $idalmoco = "S";
                    $tipo = "S";
                }
                
                $dadosInsertRegistro = array(
                        "NRREGISTROHORAS" => $codRegistroHoras, 
                        "NRAPONTAMENTO" => $nrApontamento,
                        "DTHRENTRADA" => "{$param["DATA"]} {$param["HORA"]}",
                        "DTHRSAIDA" => "",
                        "HRTEMPOREGISTRO" => ($tipo === 'S') ? null: 0,
                        "IDTIPORESGISTRO" => $tipo, 
                        "IDALMOCO" => "N",
                        "NRORG" => $NRORG, 
                        "IDATIVO" => "S",
                        "DTINCLUSAO" => $param["DATA"],
                        "DATEENTRY" => $dateEntry
                    );
                    
                $this->ApontamentoFactory->insertRegistroHoras($dadosInsertRegistro);
                $nr = $nrApontamento;
                
            } else {
                
                $arrayParamUltimoRegistro = array(
                    "DTINCLUSAO" => $param["DATA"],
                    "NRAPONTAMENTO" => $consultaApontamento[0]['NRAPONTAMENTO']
                );
            
                $ultimoRegistroInserido = $this->ApontamentoFactory->buscaUltimoRegistroInserido($arrayParamUltimoRegistro);

                if ($ultimoRegistroInserido[0]["IDTIPORESGISTRO"] === "S" && $ultimoRegistroInserido[0]["IDALMOCO"] === "S" && $param["IDTIPORESGISTRO"] === "1") {

                    $lastEntry = $this->ApontamentoFactory->lastEntry($param["CDOPERADOR"]);
                    
                    if (!empty($lastEntry)) {
                      $dateEntry = $lastEntry[0]['DTHRENTRADA'];
                    } else {
                      $dateEntry = null;
                    }
                    
                    $entrada = "";
                    $saida = "";
                    $idalmoco = "";
                    $tipo = "";
                     
                    if ($param["IDTIPORESGISTRO"] === "1") {
                        $entrada = "{$param["DATA"]} {$param["HORA"]}";
                        $idalmoco = "S";
                        $tipo = "E";
                    }
                    
                    
                     $dadosInsertRegistro = array(
                        "NRREGISTROHORAS" => $codRegistroHoras, 
                        "NRAPONTAMENTO" => $consultaApontamento[0]['NRAPONTAMENTO'],
                        "DTHRENTRADA" => $entrada,
                        "DTHRSAIDA" => $saida,
                        "HRTEMPOREGISTRO" => ($tipo === "S") ? null: 0,
                        "IDTIPORESGISTRO" => $tipo, 
                        "IDALMOCO" => $idalmoco,
                        "NRORG" => $NRORG, 
                        "IDATIVO" => "S",
                        "DTINCLUSAO" => $param["DATA"],
                        "DATEENTRY"      => $dateEntry
                    );

                    $this->ApontamentoFactory->insertRegistroHoras($dadosInsertRegistro);
                    $nr = $consultaApontamento[0]['NRAPONTAMENTO'];
                    
                } else {
                    
                    $lastEntry = $this->ApontamentoFactory->lastEntry($param["CDOPERADOR"]);
                    
                    if (!empty($lastEntry)) {
                      $dateEntry = $lastEntry[0]['DTHRENTRADA'];
                    } else {
                      $dateEntry = null;
                    }

                    $entrada = "";
                    $saida = "";
                    $idalmoco = "";
                    $tipo = "";
                     
                    if ($param["IDTIPORESGISTRO"] === "1" && $param["IDALMOCO"] === 0) {
                        $entrada = "{$param["DATA"]} {$param["HORA"]}";
                        $idalmoco = "N";
                        $tipo = "E";
                    } else if ($param["IDTIPORESGISTRO"] === "1" && $param["IDALMOCO"] === 1){
                        $entrada = "{$param["DATA"]} {$param["HORA"]}";
                        $idalmoco = "S";
                        $tipo = "E";
                    } else if ($param["IDTIPORESGISTRO"] === "2" && $param["IDALMOCO"] === 0) {
                        $saida = "{$param["DATA"]} {$param["HORA"]}";
                        $idalmoco = "N";
                        $tipo = "S";
                    } else if ($param["IDTIPORESGISTRO"] === "2" && $param["IDALMOCO"] === 1){
                        $saida = "{$param["DATA"]} {$param["HORA"]}";
                        $idalmoco = "S";
                        $tipo = "S";
                    }

                     $dadosInsertRegistro = array(
                        "NRREGISTROHORAS" => $codRegistroHoras, 
                        "NRAPONTAMENTO" => $consultaApontamento[0]['NRAPONTAMENTO'],
                        "DTHRENTRADA" => $entrada,
                        "DTHRSAIDA" => $saida,
                        "HRTEMPOREGISTRO" => ($tipo === 'S') ? null: 0,
                        "IDTIPORESGISTRO" => $tipo, 
                        "IDALMOCO" => $idalmoco,
                        "NRORG" => $NRORG, 
                        "IDATIVO" => "S",
                        "DTINCLUSAO" => $param["DATA"],
                        "DATEENTRY"      => $dateEntry
                    );
                    $this->ApontamentoFactory->insertRegistroHoras($dadosInsertRegistro);
                    $nr = $consultaApontamento[0]['NRAPONTAMENTO'];
                }
            }
            $response = "OK-$nr";
        }
        catch(\Exception $e) {
            
            $response = $e->getMessage();
        }
        
        return $response;
    }
    
    public function deleteRegistroHoras($param) {
        try {
            
            $data = $param['objeto']['data'];
            
            $quantidadeRegistrosDeHora = $this->ApontamentoFactory->quantidadeDeRegistros($data);
            
            $paramentroDeleteRegisto = array(
                "NRREGISTROHORAS" => $param['objeto']['name']
            );

            if ($quantidadeRegistrosDeHora[0]["QUANTIDIDADE"] == 1){

                $this->ApontamentoFactory->deleteRegistroHoras($paramentroDeleteRegisto);
                
                $paramentroDeleteApontamento = array(
                    "NRAPONTAMENTO" => $quantidadeRegistrosDeHora[0]["NRAPONTAMENTO"]
                );

                $this->ApontamentoFactory->deleteApontamento($paramentroDeleteApontamento);
                
            }
            else {
                
                $this->ApontamentoFactory->deleteRegistroHoras($paramentroDeleteRegisto);
                
                $parametroApuracao = array(
                    "NRAPONTAMENTO" => $quantidadeRegistrosDeHora[0]["NRAPONTAMENTO"],
                    "HRENTRADA" => '',
                    "HRSAIDA" => '',
                    "HRINTERVALO" => '',
                    "HRTRABALHADAS" => '',
                    "HRAUSENCIA" => '',
                    "HRTOTAIS" => '',
                    "IDFALTA" => 'N',
                    "IDAPURADO" => 'N'
                );
                
                $this->ApontamentoFactory->apurarHoras($parametroApuracao);
                
            }
            
            $response = "DELETADO";
        }
        catch(\Exception $e) {
             $response = $e->getMessage();
        }
        
        return $response;
        
    }
    
    public function apurarHoras($param) {
        try {
            
            $paramentroConsultaApontamento = array(
              "DTAPONTAMENTO" => $param[0]["DATA"],
              "CDOPERADOR" => $param[0]["CDOPERADOR"]
            );
                    
            $consultaApontamento = $this->ApontamentoFactory->consultaApontamentoDia($paramentroConsultaApontamento);
    
            $parametroApuracao = array(
                "NRAPONTAMENTO" => $consultaApontamento[0]["NRAPONTAMENTO"],
                "HRENTRADA" => trim($param[0]["HRENTRADA"]),
                "HRSAIDA" => trim($param[0]["HRSAIDA"]),
                "HRINTERVALO" => trim($param[0]["HRINTERVALO"]),
                "HRTRABALHADAS" => trim($param[0]["HRTRABALHADAS"]),
                "HRAUSENCIA" => trim($param[0]["HRAUSENCIA"]),
                "HRTOTAIS" => trim($param[0]["HRTOTAIS"]),
                "IDFALTA" => $param[0]["IDFALTA"],
                "IDAPURADO" => $param[0]["IDAPURADO"]
            );

            $this->ApontamentoFactory->apurarHoras($parametroApuracao);
            $response = "APURADO";

        } catch(\Exception $e) {
             $response = $e->getMessage();
        }
        return $response;
    }
    
    public function setNewRegister($parametros) {

      $NRORG =  $this->Environment->getNrOrg();
      $NRUSUARIOLOGADO = $parametros['CDOPERADOR'];

      $todayrRegister = $this->ApontamentoFactory->getTodayRegister($NRUSUARIOLOGADO);
      // debugApi([$todayrRegister]);
      $qtRegisters = $todayrRegister[0]['QTDAP'];

      $this->connection->beginTransaction();

      try {
        $NRAPONTAMENTO = null;

        if ($qtRegisters == 0) {
          $NRAPONTAMENTO = $this->newCode->getCodeByTableName("FCM_APONTAMENTO", $NRORG, 1, 10, true);
          $parametrosNewAppointment = array(
              "NRAPONTAMENTO" => $NRAPONTAMENTO,
              "CDOPERADOR" => $parametros['CDOPERADOR'],
              "DTAPONTAMENTO" => $parametros['DTAPONTAMENTO'],
              "IDFALTA"       => 'N',
              "IDAPURADO"     => 'N',
              "NRORG"         => $NRORG
          );
          $this->ApontamentoFactory->insertNewAppointment($parametrosNewAppointment);
        }

        if ($NRAPONTAMENTO == null) {
          $nr = $this->ApontamentoFactory->getNRAP($NRUSUARIOLOGADO);
          $NRAPONTAMENTO = $nr[0]['NRAPONTAMENTO'];
        }

        $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

        $lastEntry = $this->ApontamentoFactory->lastEntry($NRUSUARIOLOGADO);
        if (!empty($lastEntry)) {
          $dateEntry = $lastEntry[0]['DTHRENTRADA'];
        } else {
          $dateEntry = null;
        }
        // debugApi([$dateEntry, $parametros['DTHRSAIDA']]);

        $parametrosNewRegister = array(
              "NRREGISTROHORAS"=> $NRREGISTROHORAS,
              "NRAPONTAMENTO"  => $NRAPONTAMENTO,
              "DTHRENTRADA"    => $parametros['DTHRENTRADA'],
              "IDTIPORESGISTRO"=> $parametros['IDTIPORESGISTRO'],
              "HRTEMPOREGISTRO"=> ($parametros['IDTIPORESGISTRO'] === 'S') ? null: 0,
              "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
              "IDALMOCO"       => $parametros['IDALMOCO'],
              "DTHRSAIDA"      => $parametros['DTHRSAIDA'],
              "NRORG"          => $NRORG,
              "DATEENTRY"      => $dateEntry
          );

          $this->ApontamentoFactory->insertNewRegister($parametrosNewRegister);
          $this->connection->commit();
          $response = "OK";

      } catch(\Exception $e) {
          $this->connection->rollback();
          $response = ($e->getMessage());
      }
       return $response;
    }

    public function typeRegister($param) {
      $types = $this->ApontamentoFactory->typeRegister($param['NRFUNC']);
      return $types;
    }

    public function optionLunch($parametros) {

      $qtLunch = $this->ApontamentoFactory->optionLunch($parametros['NRFUNC']);
      $types = $this->ApontamentoFactory->typeRegister($parametros['NRFUNC']);

      $arrayOfReturn = [$qtLunch, $types];
      // debugApi([$arrayOfReturn]);
      return $arrayOfReturn;
    }

    public function cardsRegisters($param) {
      $cards = $this->ApontamentoFactory->cardsRegisters($param['NRFUNC']);
      return $cards;
    }

    public function setAbsence($parametros) {
    // debugApi([$parametros]);

      $this->connection->beginTransaction();
      $NRORG = $this->Environment->getNrOrg();

      try {
          $NRAPONTAMENTO = $this->newCode->getCodeByTableName("FCM_APONTAMENTO", $NRORG, 1, 10, true);
          $paramsAppointment = array(
              "NRAPONTAMENTO" => $NRAPONTAMENTO,
              "CDOPERADOR" => $parametros['CDOPERADOR'],
              "DTAPONTAMENTO" => $parametros['DTAPONTAMENTO'],
              "IDFALTA"       => 'S',
              "IDAPURADO"     => 'N',
              "NRORG"         => $NRORG
          );
          $this->ApontamentoFactory->insertNewAppointment($paramsAppointment);
// debugApi(['oi']);
        $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

        $parametrosNewRegister = array(
              "NRREGISTROHORAS"=> $NRREGISTROHORAS,
              "NRAPONTAMENTO"  => $NRAPONTAMENTO,
              "DTHRENTRADA"    => null,
              "IDTIPORESGISTRO"=> null,
              "HRTEMPOREGISTRO"=> null,
              "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
              "IDALMOCO"       => 'N',
              "DTHRSAIDA"      => null,
              "NRORG"          => $NRORG,
              "DATEENTRY"      => null,
          );

          $this->ApontamentoFactory->insertNewRegister($parametrosNewRegister);

          $this->connection->commit();

          $response = "OK";

      } catch(\Exception $e) {
          $this->connection->rollback();
          $response = ($e->getMessage());
      }
       return $response;
    }

    public function setLastAppointment($parametros) {

      $this->connection->beginTransaction();
      $NRORG = $this->Environment->getNrOrg();
      $NRUSUARIOLOGADO = $parametros['CDOPERADOR'];

      try {
        $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

        $lastEntry = $this->ApontamentoFactory->lastEntry($NRUSUARIOLOGADO);

        if (!empty($lastEntry)) {
          $dateEntry = $lastEntry[0]['DTHRENTRADA'];
        } else {
          $dateEntry = null;
        }

        $parametrosNewRegister = array(
              "NRREGISTROHORAS"=> $NRREGISTROHORAS,
              "NRAPONTAMENTO"  => $parametros['NRAPONTAMENTO'],
              "DTHRENTRADA"    => null,
              "IDTIPORESGISTRO"=> 'S',
              "HRTEMPOREGISTRO"=> null,
              "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
              "IDALMOCO"       => 'N',
              "DTHRSAIDA"      => $parametros['DTHRSAIDA'],
              "NRORG"          => $NRORG,
              "DATEENTRY"      => $dateEntry,
          );

          $this->ApontamentoFactory->insertNewRegister($parametrosNewRegister);

          $this->connection->commit();

          $response = "OK";

      } catch(\Exception $e) {
          $this->connection->rollback();
          $response = ($e->getMessage());
      }
       return $response;
    }
}
?>