<?php
namespace FM\Service;

use FM\Factory\FuncoesFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;


class FuncoesService {
    
    protected $FuncoesFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    
    public function __construct (EntityManager $entityManager, FuncoesFactory $FuncoesFactory, Environment $Environment, NewCode $newCode) {
        $this->FuncoesFactory               = $FuncoesFactory;
        $this->connection                   = $entityManager->getConnection();
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }

    public function getPositionsService(){
        
        return $this->FuncoesFactory->getPositionsFactory();
    }
    
    public function positionsListService(){
        $operadores = $this->FuncoesFactory->positionsListFactory();
        return $operadores;
    }
    
    public function createPositionService($nmperfilacesso, $idpermisssusuario, $idpermissstarefas, $idpermissslocais, $idpermissschecklist, $idpermisssrelatorios, $idpermisssgraficos, $idpermisssvistoria, $idpermissscalend, $iddirecionaqrcode,  $idtipodashboard) {
        $this->connection->beginTransaction();

        try {
            $nrperfilacesso = $this->newCode->getCodeByTableName('FCM_PERFILACESSO',  $this->Environment->getNrOrg(), 1, 10, true);  
        
            $params = array (
                'NRPERFILACESSO' => $nrperfilacesso,
                'NMPERFILACESSO' => $nmperfilacesso,
                'IDPERMISSSUSUARIO' => $idpermisssusuario,
                'IDPERMISSSTAREFAS' => $idpermissstarefas,
                'IDPERMISSSLOCAIS' => $idpermissslocais,
                'IDPERMISSSCHECKLIST' => $idpermissschecklist,
                'IDPERMISSSRELATORIOS' => $idpermisssrelatorios,
                'IDPERMISSSGRAFICOS' => $idpermisssgraficos,
                'IDATIVO' => 'S',
                'IDPERMISSSVISTORIA' => $idpermisssvistoria,
                'IDPERMISSSCALEND' => $idpermissscalend,
                'IDDIRECIONAQRCODE' => $iddirecionaqrcode,
                'IDTIPODASHBOARD' => $idtipodashboard,
            );

            $this->FuncoesFactory->createPositionFactory($params);
            $this->connection->commit();

        }  catch(\Exception $e) {
          $this->connection->rollback();
          throw $e;
        }
    }
    
    public function getPositionInfoService($id){
        $operadores = $this->FuncoesFactory->getPositionInfoFactory($id);
        return $operadores;
    }
    
    public function updatePositionService($nrperfilacesso, $nmperfilacesso, $idpermisssusuario, $idpermissstarefas, $idpermissslocais, $idpermissschecklist, $idpermisssrelatorios, $idpermisssgraficos, $idpermisssvistoria, $idpermissscalend, $iddirecionaqrcode, $idtipodashboard) {
        $this->connection->beginTransaction();

        try {
            $params = array (
                'NRPERFILACESSO' => $nrperfilacesso,
                'NMPERFILACESSO' => $nmperfilacesso,
                'IDPERMISSSUSUARIO' => $idpermisssusuario,
                'IDPERMISSSTAREFAS' => $idpermissstarefas,
                'IDPERMISSSLOCAIS' => $idpermissslocais,
                'IDPERMISSSCHECKLIST' => $idpermissschecklist,
                'IDPERMISSSRELATORIOS' => $idpermisssrelatorios,
                'IDPERMISSSGRAFICOS' => $idpermisssgraficos,
                'IDPERMISSSVISTORIA' => $idpermisssvistoria,
                'IDPERMISSSCALEND' => $idpermissscalend,
                'IDDIRECIONAQRCODE' => $iddirecionaqrcode,
                'IDTIPODASHBOARD' => $idtipodashboard
            );

            $this->FuncoesFactory->updatePositionFactory($params);
            $this->connection->commit();

        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function deletePositionsService($parametros){
        $cont = 0;
        $this->connection->beginTransaction();
        
        try {
            foreach($parametros['NRPERFILACESSO'] as $nr) {
                
    
                $validParam = array (
                    'NRPERFILACESSO' => $nr
                );
                $validaRelacFunc = $this->FuncoesFactory->validPositionsInRelacFuncFactory($validParam);
                $valida = $validaRelacFunc[0]['VALIDA'];

                if($valida == 0) {
                        $this->FuncoesFactory->deletePositionsFactory($validParam);
                } else {
                    $cont++;
                }
            }
            $this->connection->commit();
            return $cont;
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    // Busca qual as permissões para o usuário logado
    public function getPositionPermiss($nrfuncionario){
        $permissions = $this->FuncoesFactory->getPermissions($nrfuncionario);
        $permissions[0]['token'] = md5(strval($nrfuncionario));
        
        return $permissions;
    }
}
?>